@@ -54,7 +54,11 @@ import net.sourceforge.jnlp.JNLPFile;
 import net.sourceforge.jnlp.config.DeploymentConfiguration;
 import net.sourceforge.jnlp.runtime.JNLPClassLoader.SecurityDelegate;
 import net.sourceforge.jnlp.runtime.JNLPRuntime;
+import net.sourceforge.jnlp.security.appletextendedsecurity.AppletSecurityActions;
 import net.sourceforge.jnlp.security.appletextendedsecurity.ExecuteAppletAction;
+import net.sourceforge.jnlp.security.appletextendedsecurity.UnsignedAppletTrustConfirmation;
+import static net.sourceforge.jnlp.security.appletextendedsecurity.UnsignedAppletTrustConfirmation.getStoredAction;
+import net.sourceforge.jnlp.security.dialogs.AccessWarningPaneComplexReturn;
 import net.sourceforge.jnlp.security.dialogs.apptrustwarningpanel.AppTrustWarningPanel.AppSigningWarningAction;
 import net.sourceforge.jnlp.util.UrlUtils;
 import net.sourceforge.jnlp.util.logging.OutputController;
@@ -131,25 +135,81 @@ public class SecurityDialogs {
      * @param file the jnlp file associated with the requesting application.
      * @return true if permission was granted by the user, false otherwise.
      */
-    public static boolean showAccessWarningDialog(AccessType accessType, JNLPFile file) {
-        return showAccessWarningDialog(accessType, file, null);
+    public static boolean showAccessWarningDialogB(AccessType accessType, JNLPFile file) {
+        return showAccessWarningDialogB(accessType, file, null);
+    }
+    
+     public static boolean showAccessWarningDialogB(AccessType accessType, JNLPFile file,  final Object[] extras) {
+        Object o = showAccessWarningDialog(accessType, file, extras);
+        if (o instanceof Boolean){
+            return (Boolean) o;
+        }
+        if (o instanceof Integer){
+            return getIntegerResponseAsBoolean((Boolean)o);
+        }
+        if (o instanceof AccessWarningPaneComplexReturn){
+            return getIntegerResponseAsBoolean(((AccessWarningPaneComplexReturn)o).getRegularReturn());
+        }
+        return false;
+    }
+    
+    /**
+     * unlike showAccessWarningDialogB this is returning raw int code
+     * @param accessType type of dialogue
+     * @param file file for which thsi dialogue is built for
+     * @return return from dialogue
+     */
+     public static int showAccessWarningDialogI(AccessType accessType, JNLPFile file) {
+        Object o = showAccessWarningDialog(accessType, file, null);
+        if (o instanceof Boolean){
+            boolean b =(Boolean) o;
+            if (b){
+                return 0;
+            } else {
+                return 1;
+            }
+        }
+        if (o instanceof Integer){
+            return (Integer)o;
+        }
+        if (o instanceof AccessWarningPaneComplexReturn){
+            return ((AccessWarningPaneComplexReturn)o).getRegularReturn();
+        }
+        return 1;
+    }
+     
+      public static AccessWarningPaneComplexReturn showAccessWarningDialogComplexReturn(AccessType accessType, JNLPFile file) {
+        Object o = showAccessWarningDialog(accessType, file, null);
+        if (o instanceof AccessWarningPaneComplexReturn){
+            return (AccessWarningPaneComplexReturn)o;
+        }
+          if (o instanceof Boolean) {
+              boolean b = (Boolean) o;
+              if (b) {
+                  return new AccessWarningPaneComplexReturn(0);
+              } else {
+                  return new AccessWarningPaneComplexReturn(1);
+              }
+          }
+          if (o instanceof Integer) {
+              return new AccessWarningPaneComplexReturn((int) o);
+          }
+       return new  AccessWarningPaneComplexReturn(1);
     }
 
+                 
     /**
-     * Shows a warning dialog for different types of system access (i.e. file
-     * open/save, clipboard read/write, printing, etc).
-     *
-     * @param accessType the type of system access requested.
-     * @param file the jnlp file associated with the requesting application.
-     * @param extras an optional array of Strings (typically) that gets
-     * passed to the dialog labels.
-     * @return true if permission was granted by the user, false otherwise.
+     * unlike showAccessWarningDialogB this is returning raw int code
+     * @param accessType type of dialogue
+     * @param file file for which thsi dialogue is built for
+     * @param extras aditional parameters to dialogue
+     * @return return from dialogue
      */
-    public static boolean showAccessWarningDialog(final AccessType accessType,
+    public static Object showAccessWarningDialog(final AccessType accessType,
             final JNLPFile file, final Object[] extras) {
 
         if (!shouldPromptUser()) {
-            return false;
+            return 1;
         }
 
         final SecurityDialogMessage message = new SecurityDialogMessage();
@@ -159,15 +219,16 @@ public class SecurityDialogs {
         message.file = file;
         message.extras = extras;
 
-        Object selectedValue = getUserResponse(message);
+        return getUserResponse(message);
 
-        return getIntegerResponseAsBoolean(selectedValue);
+      
     }
 
     /**
      * Shows a warning dialog for when a plugin applet is unsigned.
      * This is used with 'high-security' setting.
      *
+     * @param file the file to be base as information source for this dialogue
      * @return true if permission was granted by the user, false otherwise.
      */
     public static AppSigningWarningAction showUnsignedWarningDialog(JNLPFile file) {
@@ -193,6 +254,7 @@ public class SecurityDialogs {
      * @param accessType the type of warning dialog to show
      * @param file the JNLPFile associated with this warning
      * @param certVerifier the JarCertVerifier used to verify this application
+     * @param securityDelegate the delegate for security atts.
      *
      * @return RUN if the user accepted the certificate, SANDBOX if the user
      * wants the applet to run with only sandbox permissions, or CANCEL if the
@@ -220,6 +282,9 @@ public class SecurityDialogs {
     /**
      * Shows a warning dialog for when an applet or application is partially signed.
      *
+     * @param file the JNLPFile associated with this warning
+     * @param certVerifier the JarCertVerifier used to verify this application
+     * @param securityDelegate the delegate for security atts.
      * @return true if permission was granted by the user, false otherwise.
      */
     public static AppSigningWarningAction showPartiallySignedWarningDialog(JNLPFile file, CertVerifier certVerifier,
@@ -288,18 +353,40 @@ public class SecurityDialogs {
         return getIntegerResponseAsBoolean(selectedValue);
     } 
      
-     public static boolean showMatchingALACAttributePanel(String title, URL codeBase, Set<URL> remoteUrls) {
+     public static boolean showMatchingALACAttributePanel(JNLPFile file, URL codeBase, Set<URL> remoteUrls) {
+         
+        ExecuteAppletAction storedAction = getStoredAction(file, AppletSecurityActions.MATCHING_ALACA_ACTION);
+        OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Stored action for matching alaca at " + file.getCodeBase() +" was " + storedAction);
+        
+        if (storedAction != null){
+            if (storedAction == ExecuteAppletAction.ALWAYS){
+                return true;
+            }
+            if (storedAction == ExecuteAppletAction.NEVER){
+                return false;
+            }
+        }
+        
 
         if (!shouldPromptUser()) {
             return false;
         }
 
-        SecurityDialogMessage message = new SecurityDialogMessage();
-        message.dialogType = DialogType.MATCHING_ALACA;
-        message.extras = new Object[]{title, codeBase.toString(), UrlUtils.setOfUrlsToHtmlList(remoteUrls)};
-        Object selectedValue = getUserResponse(message);
-        return getIntegerResponseAsBoolean(selectedValue);
-    } 
+         SecurityDialogMessage message = new SecurityDialogMessage();
+         message.dialogType = DialogType.MATCHING_ALACA;
+         message.extras = new Object[]{file, codeBase.toString(), UrlUtils.setOfUrlsToHtmlList(remoteUrls)};
+         AppSigningWarningAction selectedValue = (AppSigningWarningAction) getUserResponse(message);
+
+         if (selectedValue != null) {
+             OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for matching alaca at " + file.getCodeBase() + " was " + selectedValue.getAction());
+             UnsignedAppletTrustConfirmation.updateAppletAction(file, selectedValue.getAction(), selectedValue.rememberForCodeBase(), AppletSecurityActions.MATCHING_ALACA_ACTION);
+             return selectedValue.getAction() == ExecuteAppletAction.ALWAYS || selectedValue.getAction() == ExecuteAppletAction.YES;
+         }
+
+         OutputController.getLogger().log(OutputController.Level.ERROR_DEBUG, "Decided action for matching alaca at " + file.getCodeBase() + " was " + null);
+         return false;
+
+    }
      
     /**
      * FIXME This is unused. Remove it?
@@ -319,7 +406,7 @@ public class SecurityDialogs {
         // result 0 = Yes, 1 = No, 2 = Cancel
         if (selectedValue instanceof Integer) {
             // If the selected value can be cast to Integer, use that value
-            return ((Integer) selectedValue).intValue();
+            return ((Integer) selectedValue);
         } else {
             // Otherwise default to "cancel"
             return 2;
@@ -428,7 +515,7 @@ public class SecurityDialogs {
         boolean isInteger = ref instanceof Integer;
         if (isInteger) {
             Integer i = (Integer) ref;
-            return i.intValue() == 0;
+            return i == 0;
         }
         return false;
     }
