@@ -23,35 +23,18 @@ public class DummyJNLPFileWithJar extends JNLPFile {
     }
 
     private final JARDesc[] jarDescs;
-    private final URL[] jarFiles;
+    private final File[] jarFiles;
 
     public DummyJNLPFileWithJar(File... jarFiles) throws MalformedURLException {
         this(-1, jarFiles);
     }
-    
-    public DummyJNLPFileWithJar(URL codebaseRewritter, URL... jarFiles) throws MalformedURLException {
-        this(-1, codebaseRewritter, jarFiles);
-    }
     public DummyJNLPFileWithJar(int main, File... jarFiles) throws MalformedURLException {
-        this(main, jarFiles[0].getParentFile().toURI().toURL(), filesToUrls(jarFiles));
-
-    }
-    
-    private static URL[] filesToUrls(File[] f) throws MalformedURLException{
-        URL[] r = new URL[f.length];
-        for (int i = 0; i < f.length; i++) {
-            r[i]=f[i].toURI().toURL();
-        }
-        return r;
-    }
-    
-    public DummyJNLPFileWithJar(int main, URL codebaseRewritter, URL... jarFiles) throws MalformedURLException {
-        codeBase = codebaseRewritter;
+        codeBase = jarFiles[0].getParentFile().toURI().toURL();
         this.jarFiles = jarFiles;
         jarDescs = new JARDesc[jarFiles.length];
 
         for (int i = 0; i < jarFiles.length; i++) {
-            jarDescs[i] = makeJarDesc(jarFiles[i], i==main);
+            jarDescs[i] = makeJarDesc(jarFiles[i].toURI().toURL(), i==main);
 
         }
         info = new ArrayList<InformationDesc>();
@@ -59,11 +42,19 @@ public class DummyJNLPFileWithJar extends JNLPFile {
     }
 
     public URL getJarLocation() {
-            return jarFiles[0];
+        try {
+            return jarFiles[0].toURI().toURL();
+        } catch (MalformedURLException e) {
+            throw new RuntimeException(e);
+        }
     }
     
     public URL getJarLocation(int i) {
-            return jarFiles[i];
+        try {
+            return jarFiles[i].toURI().toURL();
+        } catch (MalformedURLException e) {
+            throw new RuntimeException(e);
+        }
     }
 
     public JARDesc[] getJarDescs() {
@@ -97,11 +88,6 @@ public class DummyJNLPFileWithJar extends JNLPFile {
         return codeBase;
     }
 
-    @Override
-    public SecurityDesc getSecurity() {
-        return new SecurityDesc(this, SecurityDesc.SANDBOX_PERMISSIONS, null);
-    }
-
     public void setInfo(List<InformationDesc> info) {
         this.info = info;
     }
